/* Copyright (C) 1999 Jeff Heflin - University of Maryland
 *	
 * This software is provided under the terms of the GNU Lesser General Public
 * License Version 2.1, February 1999. See the header in ShoeKb.java
 * and the file License.html for details.
 */

/* Change History:
   - 10/06/00, JDH: Removed getQueryVars() because it has been replaced by
                    Sentence.getVariables()
   - 05/22/00, JDH: Use arrayCopy() in getQueryVars()
   */
package ShoeKb;

import java.util.Vector;
import java.util.Enumeration;

/** An abstract class that specifies methods necessary for communicating 
  with a knowledge base that stores SHOE data. */
public abstract class KBInterface {

  /* These are different from the ShoeKb constants b/c they don't need
     to overide being placed in a context. */

  /** A binary predicate that states the first argument is a subcategory of
    the second argument. */
  public static final String P_ISA            = "isa";

  /** A binary predicate that states the second argument is explicitly
    a member of the category in the first argument. */
  public static final String P_INSTANCE       = "instance";

  /** A binary predicate that states the second argument is a member of
    the category in the first argument either explictly or by the
    transitivity of category membership. This predicate should only
    be used in queries.*/
  public static final String P_EVERY_INSTANCE = "everyInstance";

  /** A binary predicate that is true if the the first argument matches
    the second argument when percent signs ('%') in the second argument
    are interpreted as wildcard characters. */
  public static final String P_STRING_MATCH   = "stringMatch";

  /** Establish a connection to the given host and port number. */
  public abstract void establishConnection(String host, int port)
       throws KBException;

  /** Creates a new KB with the name supplied by the user. */
  public abstract void createKB(String kbName)
       throws KBException;

  /** Opens an existing knowledge base that is avaiable through the
    given host and port number. */
  public abstract void openKB(String kbName, String host, int port)
       throws KBException;

  /** Opens an existing knowledge base using the default location. */
  public abstract void openKB(String kbName)
       throws KBException;

  /** Closes the current knowledge base. */
  public abstract void closeKB();

  /** Closes the current connection. Any open KBs will be closed as well. */
  public abstract void closeConnection();
       
  /** Issues an arbitrary query to the KB. Variable arguments are 
    preceded by question marks. The answers are returned in a matrix. */
  public abstract ResultSet queryKB(Query query);
       
  /** Stores a claim consisting of the predicate and array of arguments
    and attributes it to the given source. */
  /* NOTE: We must decide whether it makes sense to have a storeClaim()
     method in a generic KB class. A storeAssertion() method would
     probably be more appropriate, with storeClaim() moved to ShoeKb. */
  /* NOTE: To make this more generic we might decide to replace the
     predicate and args inputs with a sentence input. */
  public abstract void storeClaim(String source, String predicate, String args[]);

  /** Deletes all claims made by the source. */
  public abstract void deleteClaims(String source);

  /** Creates a new class with the specified name. */
  public abstract void createClass(String className);

  /** Creates a new predicate with given name and arity. */
  public abstract void createPredicate(String predicate, int arity);

  /** Creates a new predicate with given name and types for
      arguments. */
  public abstract void createPredicate(String predicate, String argTypes[]);

  /** Asserts a new axiom to the KB. */
  public abstract void storeAxiom(Axiom anAxiom);


  /* ************* Non-abstract methods ************** */

  /** Issues a query to the KB where the given arguments are applied
    to the given predicate. A variable argument must be preceded by a
    question mark. */
  public ResultSet queryKB(String predicate, String args[]) {

    Query     qry;
    ResultSet answers= null;

    qry = new Query(predicate, args);
    answers = queryKB(qry);
    return answers;
  }

  /** Issues a query to the KB where the two given arguments are applied
    to the given predicate. A variable argument must be preceded by a
    question mark. */
  public ResultSet queryKB(String predicate, String arg1, String arg2) {

    String[]  args;
    ResultSet answers;

    args = new String[2];
    args[0] = arg1;
    args[1] = arg2;
    answers = queryKB(predicate, args);
    return answers;
  }


  /** Stores a claims for a binary predicate and attributes it to the
    given source. */
  public void storeClaim(String source, String predicate,
			 String arg1, String arg2) {

    String args[];

    args = new String[2];
    args[0] = arg1;
    args[1] = arg2;
    storeClaim(source, predicate, args);
  }


  /* ***** Protected methods ********* */

  /** Given a query, returns an array of the query's variables, in the
      order in which they appear in the query. This is meant to be
      a helper function for KB interfaces to determine what order the
      variable bindings should be returned in.*/ 
  /*
  protected String[] getQueryVars(Query query) {

    Vector      varVector;
    String[]    varArray;
    Enumeration enum;
    Atom        atom;
    String[]    args;
    int         i, j;
    boolean     found;

    varVector = new Vector();
    enum = query.getAtoms().elements();
    while (enum.hasMoreElements()) {
      atom = (Atom)enum.nextElement();
      args = atom.getArguments();
      for (i=0; i < args.length; i++) {
	if (args[i].startsWith("?")) {

	  // see if variable is already in list
	  j = 0;
	  found = false;
	  while (j < varVector.size() && found == false) {
	    if (args[i].equals(varVector.elementAt(j)))
	      found = true;
	    j++;
	  }

	  if (found == false) {
	    varVector.addElement(args[i]);
	  }
	}
      }
    }
    varArray = new String[varVector.size()];
    varVector.copyInto(varArray);
    return varArray;
  }
  */
}






